unit PointSets;
  { ADT for 2D point sets }
  { Tom Verhoeff (TUE, 2007) }

// Please do not remove the BEGIN-END TODO markers

//# BEGIN TODO Completed by: author name, id.nr., date
  Replace this line by your text.
//# END TODO

interface

uses
  Vectors;

type
  PNode = ^TNode;
  TNode = // node of binary tree over TVector
  record
    FVector: TVector; // value stored in this node
    FLeft: PNode; // left subtree
    FRight: PNode; // right subtree
  end;
  TBinTree = PNode; // Binary Trees
    // data invariants for t: TBinTree
    //   I0: all root paths of t via FLeft/FRight terminate in nil
    //   I1: if two root paths of t both end in u then u = nil
    // Abstraction function for t: TBinTree
    //   empty tree if t=nil else <t^.FLeft, t^.FVector, t^.FRight>

    // ListOf(t: TBinTree) is defined recursively by
    // (Base) [ ], if t = nil (empty list)
    // (Step) ListOf(t^.FLeft) ++ [t^.FVector] ++ ListOf(t^.FRight),
    //        if t <> nil

    // SetOf(t: TBinTree) is defined recursively by
    // (Base) [ ], if t = nil (empty set)
    // (Step) SetOf(t^.FLeft) Union [t^.FVector] Union SetOf(t^.FRight),
    //        if t <> nil

  TBST = TBinTree; // Binary Search Tree
    // data invariants for t: TBST
    //   I2: ListOf(t) is strictly increasing

  TAction = procedure(AVector: TVector); // for VistBT
    // N.B. This must be a "normal" procedure, not a class method.
    // Good: procedure DrawLayer1(AVector: TVector);
    // Bad : procedure TForm1.DrawLayer1(AVector: TVector);

// operations on Binary Trees (exposed on interface for testing)
function MakeNodeBT(AVector: TVector; L, R: TBinTree): TBinTree;
  // pre: L and R do not share any nodes
  // ret: <L, AVector, R>
function HeightBT(t: TBinTree): Integer;
  // pre: true
  // ret: height of t
procedure DisposeBT(var t: TBinTree);
  // pre: true
  // post: all nodes of t and the vectors they contain have been freed
procedure InOrderBT(t: TBinTree; AAction: TAction);
  // pre: Assigned(AAction)
  // post: AAction has been applied to each node of t in in-order

// the next one is optional; if it is not implemented
// then do Count, Result := 0, 0
procedure TotalRootDistanceBT(t: TBinTree; var Count, Result: Integer);
  // pre: true
  // post: Count = #.SetOf(t) /\
  //       Result = (sum x: x node of t: distance(root of t, x))

// operations on Binary Search Trees (exposed on interface for testing)
procedure InsertBST(AVector: TVector; var t: TBST; var c: Integer);
  // pre: SetOf(t) = T /\ c = C
  // post: SetOf(t) = T Union [ AVector ] /\
  //    c = C + #.t - #.T
  //    that is, c is incremented by 1 iff AVector not in T

type
  TPointSet = class(TObject) // Abstract Data Type
  private
    FTree: TBST; // holds the set of TVector
    FCount: Integer; // # nodes in FTree

    // private invariants -------------
    // Pri0: FCount = #.Abstr

    // representation -----------------
    // Abstr = SetOf(FTree)
  public
    // construction -------------------
    constructor Create;
      // pre: true
      // post: Abstr = [ ]
    destructor Destroy; override;
      // pre: true
      // post: all memory for Self has been freed

    // queries ------------------------
    function Count: Integer;
      // pre: true
      // ret: #.Abstr
    procedure Visit(AAction: TAction);
      // pre: true
      // post: AAction has been applied once to each x in Abstr

    // commands -----------------------
    procedure Add(AVector: TVector);
      // pre: Abstr = S
      // post: Abstr = S Union [ AVector ]
    procedure Clear;
      // pre: true
      // post: Abstr = [ ]

    // model variables ----------------
    // Abstr: set of TVector

    // public invariants --------------
    // none
  end;

// auxiliary functions for TPointSet
function TreeHeight(APointSet: TPointSet): Integer;
  // pre: APointSet <> nil
  // ret: height of APointSet.FTree

// the next one is optional; if it is not implemented do Result := 0.0
function AverageRootDistance(APointSet: TPointSet): Real;
  // pre: APointSet <> nil
  // ret: average root distance over all nodes in APointSet.FTree,
  //   that is, Total Root Distance divided by Number of Nodes if FTree <> nil
  //   else 0.0

implementation

uses
  Math; // for Max

function MakeNodeBT(AVector: TVector; L, R: TBinTree): TBinTree;
begin
  // pre not checked, because that is too costly/complicated
//# BEGIN TODO body of MakeNodeBT
  Replace this line by your text.
//# END TODO
end;

function HeightBT(t: TBinTree): Integer;
begin
//# BEGIN TODO body of HeightBT
  Replace this line by your text.
//# END TODO
end;

procedure DisposeBT(var t: TBinTree);
begin
//# BEGIN TODO body of DisposeBT
  Replace this line by your text.
//# END TODO
end;

procedure InOrderBT(t: TBinTree; AAction: TAction);
begin
  // pre not checked, because that is too costly under recursion
//# BEGIN TODO body of InOrderBT
  Replace this line by your text.
//# END TODO
end;

procedure TotalRootDistanceBT(t: TBinTree; var Count, Result: Integer);
var
//# BEGIN TODO local variables of TotalRootDistanceBT
  Replace this line by your text.
//# END TODO
begin
  Count := 0; // default value
  Result := 0; // default value
//# BEGIN TODO body of TotalRootDistanceBT
  Replace this line by your text.
//# END TODO
end;

procedure InsertBST(AVector: TVector; var t: TBST; var c: Integer);
begin
//# BEGIN TODO body of InsertBST
  Replace this line by your text.
//# END TODO
end;

{ TPointSet }

constructor TPointSet.Create;
begin
  inherited Create;
//# BEGIN TODO body of TPointSet.Create
  Replace this line by your text.
//# END TODO
end;

destructor TPointSet.Destroy;
begin
  DisposeBT(FTree);
  inherited Destroy;
end;

function TPointSet.Count: Integer;
begin
//# BEGIN TODO body of TPointSet.Count
  Replace this line by your text.
//# END TODO
end;

procedure TPointSet.Add(AVector: TVector);
begin
//# BEGIN TODO body of TPointSet.Add
  Replace this line by your text.
//# END TODO
end;

procedure TPointSet.Clear;
begin
//# BEGIN TODO body of TPointSet.Clear
  Replace this line by your text.
//# END TODO
end;

procedure TPointSet.Visit(AAction: TAction);
begin
  Assert(Assigned(AAction),
    'pre.TPointSet.Visit failed: AAction not assigned');
//# BEGIN TODO body of TPointSet.Visit
  Replace this line by your text.
//# END TODO
end;

function TreeHeight(APointSet: TPointSet): Integer;
begin
//# BEGIN TODO body of TreeHeight
  Replace this line by your text.
//# END TODO
end;

function AverageRootDistance(APointSet: TPointSet): Real;
var
//# BEGIN TODO local variables of AverageRootDistance
  Replace this line by your text.
//# END TODO
begin
  Result := 0.0; // default value
//# BEGIN TODO body of AverageRootDistance
  Replace this line by your text.
//# END TODO
end;

end.
